@@ -1,9 +1,10 @@
 from decimal import Decimal
-from typing import Union, Literal, List, Optional, Any, Set
 
+from typing import Union, Literal, List, Optional, Any, Set
 from pydantic import Field, model_validator
 
 from ..common import *
+from ..sg_finetune import *
 
 
 class ServicePlan(str, Enum):
@@ -73,6 +74,11 @@ class CreateJobOut(CommonOut):
     info: Optional[Any] = None
 
 
+class CheckFtIn(BaseModel):
+    status_code: int
+    job_out: CreateJobOut
+
+
 class gDriveOut(BaseModel):
     success: bool
     refresh_token: Optional[Any] = None
@@ -127,6 +133,32 @@ class User(BaseModel):
         arbitrary_types_allowed = True
 
 
+class OrderType(str, Enum):
+    SUBSCRIPTION = "SUBSCRIPTION"
+    OTP = "OTP"
+
+
+class CreateSubscriptionInput(BaseModel):
+    amount: int
+    months: int
+    payment_method_type: str
+
+
+class TopUpRegularBalanceInput(BaseModel):
+    amount: int
+    payment_method_type: str
+
+
+class UserSubscription(BaseModel):
+    user_id: str
+    active: bool
+    discount_balance: Decimal
+    regular_balance: Decimal
+    refill_per_month: Decimal
+    months_left: int
+    last_refill: str
+
+
 class SecretOut(BaseModel):
     secret: Dict[str, Any]
     success_status: bool
@@ -279,6 +311,24 @@ class EventDB(Event):
     pass
 
 
+class DiscountCondition(str, Enum):
+    equal = "EQUAL"
+    gt = "GT"
+    gte = "GTE"
+    lt = "LT"
+    lte = "LTE"
+    not_equal = "NOT_EQUAL"
+
+
+class Discount(BaseModel):
+    id: str
+    condition: DiscountCondition
+    field: str
+    valid_until: str
+    value: Union[str, int]
+    discount: Decimal
+
+
 class NodeUsage(BaseModel):
     id: str
     entity_id: str
@@ -417,17 +467,29 @@ class InferenceDeploymentInitialWorkersConfig(BaseModel):
     on_prem_node_ids: Optional[List[str]] = None
 
 
+class InferenceDeploymentAutoscalingStrategy(str, Enum):
+    ttft_latency_sec = "ttft_latency_sec"
+    rps_per_worker = "rps_per_worker"
+
+
 class InferenceDeploymentAutoscalingConfig(BaseModel):
     scale_up_time_window_sec: Optional[int] = 5 * 60
     scale_down_time_window_sec: Optional[int] = 5 * 60
-    upper_allowed_latency_sec: Optional[float] = 1
-    lower_allowed_latency_sec: Optional[float] = 0.2
     scaling_up_timeout_sec: Optional[int] = 20 * 60
     scaling_down_timeout_sec: Optional[int] = 20 * 60
     scale_to_zero_timeout_sec: Optional[int] = 30 * 60
     enable_speedup_shared: Optional[bool] = False
     enable_fast_autoscaling: Optional[bool] = False
     scale_to_zero: Optional[bool] = False
+    autoscaling_strategy: InferenceDeploymentAutoscalingStrategy = (
+        InferenceDeploymentAutoscalingStrategy.ttft_latency_sec
+    )
+    upper_allowed_threshold: Optional[float] = 1
+    lower_allowed_threshold: Optional[float] = 0.2
+
+    # For backward compatibility. Should be deleted in the future
+    upper_allowed_latency_sec: Optional[float] = 1
+    lower_allowed_latency_sec: Optional[float] = 0.2
 
 
 class InferenceDeploymentAPIGatewayData(BaseModel):
@@ -465,7 +527,7 @@ class InferenceControllerOnPremConfig(BaseModel):
 class LLMLoraConfig(BaseModel):
     name: str
     hf_repo: str
-    hf_token: Optional[str]
+    hf_token: Optional[str] = None
 
 
 class InferenceDeploymentIn(BaseModel):
@@ -489,7 +551,7 @@ class InferenceDeploymentIn(BaseModel):
     controller_cloud_config: Optional[InferenceControllerCloudConfig] = None
     controller_on_prem_config: Optional[InferenceControllerOnPremConfig] = None
 
-    llm_loras: Optional[LLMLoraConfig] = None
+    llm_loras: Optional[List[LLMLoraConfig]] = []
 
     def __init__(self, **data: Any):
         # Validate that controller_cloud_config or controller_on_prem_config is provided
@@ -576,8 +638,8 @@ class InferenceDeploymentDB(InferenceDeploymentIn):
     id: str
     user_id: str
     status: InferenceDeploymentStatus
-    current_price_per_hour: float
-    cost: Decimal = Decimal(0.00)
+    current_price_per_hour: float = 0
+    cost: float = 0
     last_cost_updated_time: Optional[str] = None
     timestamp: str
     link: Optional[str] = None
@@ -608,61 +670,33 @@ class NodeIP(BaseModel):
     tailscale_ip: Optional[str] = None
 
 
-# Finetuning
-
-
-class AutotrainParams(BaseModel):
-    use_peft: Optional[Literal["lora", "adalora", "ia3", "llama_adapter"]] = None
-    quantization: Optional[Literal["fp4", "nf4", "int8"]] = None
-    mixed_precision: Optional[Literal["fp16", "bf16"]] = None
-    disable_gradient_checkpointing: Optional[bool] = None
-    use_flash_attention_2: Optional[bool] = None
-    lora_r: Optional[int] = None
-    lora_alpha: Optional[int] = None
-    lora_dropout: Optional[float] = None
-    lr: float = Field(0.00003, title="Learning rate")
-    batch_size: int = Field(1, title="Batch size for training")
-    epochs: int = Field(1, title="Number of training epochs")
-    train_subset: Optional[str] = None
-    valid_split: Optional[str] = None
-    text_column: Optional[str] = None
-    gradient_accumulation: Optional[int] = None
-    max_model_length: Optional[int] = Field(
-        2048, title="Model input max context length"
-    )
-    block_size: str = Field("-1", title="Block size")
-    torch_dtype: Optional[Literal["auto", "bfloat16", "float16", "float32"]] = Field(
-        None, title="Load the model under this dtype"
-    )
-    seed: Optional[int] = None
+# FINETUNE DATAMODELS
+class FinetuningType(str, Enum):
+    CLM = "CLM"
+    SENT_TRANSFORMERS = "SENT_TRANSFORMERS"
+    SEQ2SEQ = "SEQ2SEQ"
+    TEXT_CLASSIFICATION = "TEXT_CLASSIFICATION"
+    TOKEN_CLASSIFICATION = "TOKEN_CLASSIFICATION"
 
 
 class FinetuningIn(BaseModel):
     job_name: Optional[str] = None
+    ft_type: FinetuningType = FinetuningType.CLM
     model: str
+    base_model: str
     use_recipes: Optional[bool] = False
+    artifacts_storage: Optional[str] = None
 
     data_path: str
     user_dataset: Optional[str] = None
-    hf_token: Optional[str] = None
-    project_name: Optional[str] = Field(None, title="Project name in HF hub")
-
-    push_to_hub: Optional[bool] = False
-    username: Optional[str] = None
-    repo_id: Optional[str] = Field(None, title="Repo id")
 
     use_spot: bool = False
     cloud_providers: Optional[List[CloudProviderChoice]] = []
-
-    wandb_key: Optional[str] = None
-    artifacts_storage: Optional[str] = None
-
-    autotrain_params: Optional[AutotrainParams] = None
-
-    # GPU Config
     gpu_type: Optional[GPUType] = None
     gpu_count: Optional[int] = None
 
+    autotrain_params: Optional[Union[LLMTrainingParams, SGFinetuneParams]] = None
+
 
 class FinetuningJobDB(FinetuningIn):
     id: str
@@ -674,7 +708,7 @@ class FinetuningRecipe(BaseModel):
     model: str
     gpu_type: GPUType
     gpu_count: int
-    autotrain_params: AutotrainParams
+    autotrain_params: SGFinetuneParams
 
 
 class CheckKeyIn(BaseModel):
@@ -700,7 +734,7 @@ class OpenaiScalegenIntegration(BaseModel):
     use_spot: bool = False
     cloud_providers: Optional[List[CloudProviderChoice]] = []
     wandb_key: Optional[str] = None
-    autotrain_params: Optional[AutotrainParams] = None
+    autotrain_params: Optional[SGFinetuneParams] = None
 
 
 class OpenaiWandbIntegration(
@@ -925,28 +959,35 @@ class AvailableGPUMachine(BaseModel):
     availability: List[GPUMachineAvailability]
 
 
+class GPUMachineDockerPortConfig(BaseModel):
+    host_port: int
+    container_port: int
+
+
+class GPUMachineDockerConfig(BaseModel):
+    image: str
+    args: Optional[str] = None
+    env_vars: Optional[Dict[str, str]] = None
+    ports: Optional[List[GPUMachineDockerPortConfig]] = []
+
+
 class GPUMachineCreateIn(BaseModel):
-    num_gpus: int
-    gpu_type: GPUType
-
-    instance_type: str
-    cloud: str
-    provider_reference: str
-
-    hourly_price: float
-    reference_hourly_price: float
-
-    availability: GPUMachineAvailability
-
-    virtual_mount_name: Optional[str] = None
+    machine_avail_id: str
     artifacts_store_name: Optional[str] = None
+    docker_config: Optional[GPUMachineDockerConfig] = None
+
+
+class GPUMachineStatus(str, Enum):
+    CREATING = "CREATING"
+    ACTIVE = "ACTIVE"
+    DELETING = "DELETING"
+    DELETED = "DELETED"
+    FAILED = "FAILED"
 
 
 class GPUMachineDB(GPUMachineCreateIn):
     id: str
     user_id: str
-    status: str
+    status: GPUMachineStatus
     timestamp: str
-
-    ssh_ip: str = ""
-    ssh_port: int = 22
+    instance_details: InstancePricingDB
