@@ -290,6 +290,8 @@ def create(**finetune_kwargs: Any):
             "batch_size": finetune_kwargs["batch_size"],
             "epochs": finetune_kwargs["epochs"],
             "train_subset": finetune_kwargs["train_subset"],
+            "valid_subset": finetune_kwargs["valid_subset"],
+            "train_split": finetune_kwargs["train_split"],
             "valid_split": finetune_kwargs["valid_split"],
             "text_column": finetune_kwargs["text_column"],
         },
