@@ -227,25 +227,21 @@ def verify_cloud_config_options(kwargs: Dict[str, Any]):
 
 
 def verify_lora_options(kwargs: Dict[str, Any]):
-    args = [kwargs["lora_adapter_repo"], kwargs["lora_adapter_name"]]
-    if any(args):
-        if not all(args):
-            click.echo(
-                click.style(
-                    "Error: lora_adapter_name and lora_adapter_repo must be provided together",
-                    fg="red",
-                ),
-                err=True,
-            )
-            exit()
-    else:
-        return None
+    name_token_dict = {}
+    for lora_name, hf_token in kwargs["lora_adapter_hf_token"]:
+        name_token_dict[lora_name] = hf_token
 
-    return LLMLoraConfig(
-        name=kwargs["lora_adapter_name"],
-        hf_repo=kwargs["lora_adapter_repo"],
-        hf_token=kwargs.get("lora_adapter_hf_token"),
-    )
+    lora_configs = []
+    for lora_name, repo in kwargs["lora_adapter"]:
+        lora_configs.append(
+            LLMLoraConfig(
+                name=lora_name,
+                hf_repo=repo,
+                hf_token=name_token_dict.get(lora_name, None),
+            )
+        )
+
+    return lora_configs
 
 
 @click.group(name="infer", chain=True)
@@ -588,25 +584,21 @@ def create_inf_dep(
 )
 # ******************************* OPTIMAL WORKER CONFIGURATION OPTIONS *******************************
 @click.option(
-    "--lora_adapter_repo",
-    type=click.STRING,
+    "--lora_adapter",
+    "-la",
+    type=(str, str),
     required=False,
-    help="Specify the hf repo name of the lora adapter to use",
-    show_default=True,
-)
-@click.option(
-    "--lora_adapter_name",
-    type=click.STRING,
-    required=False,
-    help="Specify the name for the lora adapter",
+    help="Specify the lora name and hf repo of the lora adapter to use, ex: --lora_adapter example_name <hf_repo>",
     show_default=True,
+    multiple=True,
 )
 @click.option(
     "--lora_adapter_hf_token",
-    type=click.STRING,
+    type=(str, str),
     required=False,
-    help="Specify the HF Hub token for the lora adapter repo",
+    help="Specify the HF Hub token for the lora adapter repo, usage ex: --lora_adapter_hf_token example_name <hf_token>",
     show_default=True,
+    multiple=True,
 )
 @click.option("-f", "--force", is_flag=True)
 @click.option("-q", "--quiet", is_flag=True)
